@@ -31,10 +31,11 @@ using System.Collections;
 using System.Text;
 using System.Xml.Serialization;
 
-using BU = Beagle.Util;
+using Beagle.Util;
 
 namespace Beagle {
 
+	[Flags]
 	public enum QueryDomain {
 		Local        = 1,
 		System       = 2,
@@ -52,6 +53,15 @@ namespace Beagle {
 		private ArrayList hitTypes = new ArrayList ();
 		private ArrayList searchSources = new ArrayList ();
 
+		private ArrayList exact_text = null;
+		private ArrayList stemmed_text = null;
+
+		private QueryPart_Or mime_type_part = null;
+		private QueryPart_Or hit_type_part = null;
+		private QueryPart_Or source_part = null;
+
+		private bool is_index_listener = false;
+
 		// Events to make things nicer to clients
 		public delegate void HitsAdded (HitsAddedResponse response);
 		public event HitsAdded HitsAddedEvent;
@@ -62,17 +72,13 @@ namespace Beagle {
 		public delegate void Finished (FinishedResponse response);
 		public event Finished FinishedEvent;
 
-		public delegate void Cancelled (CancelledResponse response);
-		public event Cancelled CancelledEvent;
-
-
 		public Query () : base (true)
 		{
 			this.RegisterAsyncResponseHandler (typeof (HitsAddedResponse), OnHitsAdded);
 			this.RegisterAsyncResponseHandler (typeof (HitsSubtractedResponse), OnHitsSubtracted);
 			this.RegisterAsyncResponseHandler (typeof (FinishedResponse), OnFinished);
-			this.RegisterAsyncResponseHandler (typeof (CancelledResponse), OnCancelled);
 			this.RegisterAsyncResponseHandler (typeof (ErrorResponse), OnError);
+			this.RegisterAsyncResponseHandler (typeof (SearchTermResponse), OnSearchTerms);
 		}
 
 		public Query (string str) : this ()
@@ -106,14 +112,6 @@ namespace Beagle {
 				this.FinishedEvent (response);
 		}
 
-		private void OnCancelled (ResponseMessage r)
-		{
-			CancelledResponse response = (CancelledResponse) r;
-
-			if (this.CancelledEvent != null)
-				this.CancelledEvent (response);
-		}
-
 		private void OnError (ResponseMessage r)
 		{
 			ErrorResponse response = (ErrorResponse) r;
@@ -121,133 +119,58 @@ namespace Beagle {
 			throw new ResponseMessageException (response);
 		}
 
+		private void OnSearchTerms (ResponseMessage r)
+		{
+			SearchTermResponse response = (SearchTermResponse) r;
+			ProcessSearchTermResponse (response);
+		}
+
 		///////////////////////////////////////////////////////////////
 
+		// This is exposed for the benefit of QueryDriver.DoQueryLocal
+		public void ProcessSearchTermResponse (SearchTermResponse response)
+		{
+			exact_text = response.ExactText;
+			stemmed_text = response.StemmedText;
+		}
+
+		///////////////////////////////////////////////////////////////
+
+		// Warning: For the moment, the daemon is allowed to IGNORE
+		// index listener queries at its discretion... so don't assume
+		// that they will work for you!  Listener queries should only be
+		// used for debugging and testing.
+
+		public bool IsIndexListener {
+			set { is_index_listener = value; }
+			get { return is_index_listener; }
+		}
+
+		///////////////////////////////////////////////////////////////
+
+		public void ClearParts ()
+		{
+			if (parts != null)
+				parts.Clear ();
+		}
+
 		public void AddPart (QueryPart part)
 		{
-			parts.Add (part);
-		}
-
-		public void AddTextRaw (string str)
-		{
-			QueryPart part = new QueryPart ();
-			part.Target = QueryPart.TargetAll;
-			part.Text = str;
-
-			parts.Add (part);
-		}
-
-		private QueryPart BiteOffPart (ref string str)
-		{
-			if (str == null || str.Length == 0)
-				return null;
-
-			int i = 0;
-
-			// Skip whitespace
-			while (i < str.Length && Char.IsWhiteSpace (str [i]))
-				++i;
-			if (i >= str.Length)
-				return null;
-
-			bool is_required = false;
-			bool is_prohibited = false;
-
-			if (str [i] == '+') {
-				is_required = true;
-				++i;
-			} else if (str [i] == '-') {
-				is_prohibited = true;
-				++i;
-			}
-
-			// Skip any whitespace after the + or -
-			while (i < str.Length && Char.IsWhiteSpace (str [i]))
-				++i;
-			if (i >= str.Length)
-				return null;
-
-			string chunk = null;
-			if (str [i] == '"') {
-				int j = str.IndexOf ('"', i+1);
-				if (j == -1) {
-					if (i+1 < str.Length)
-						chunk = str.Substring (i+1);
-					str = null;
-				} else {
-					if (j-i-1 > 0)
-						chunk = str.Substring (i+1, j-i-1);
-					str = str.Substring (j+1);
-				}
-			} else {
-				int j = str.IndexOf (' ', i);
-				if (j == -1) {
-					chunk = str.Substring (i);
-					str = null;
-				} else {
-					chunk = str.Substring (i, j-i);
-					str = str.Substring (j);
-				}
-			}
-
-			// If this happens, we just bit off an empty query
-			// (i.e. something like "").  If this happens, just
-			// try again with the new value of str.
-			if (chunk == null && str != null)
-				return BiteOffPart (ref str);
-			
-			// Otherwise we assemble the QueryPart.
-			QueryPart part = new QueryPart ();
-			part.Target = QueryPart.TargetAll;
-			part.Text = chunk;
-			part.IsRequired = is_required;
-			part.IsProhibited = is_prohibited;
-
-			return part;
+			if (part != null)
+				parts.Add (part);
 		}
 
+		// This is a human-entered query string that will be parsed in
+		// the daemon.
 		public void AddText (string str)
 		{
-			// Parse our magic little query language.
-			QueryPart part;
-			while ((part = BiteOffPart (ref str)) != null)
-				AddPart (part);
+			QueryPart_Human part;
+			part = new QueryPart_Human ();
+			part.QueryString = str;
+			AddPart (part);
 		}
 
-		// FIXME: Since it is possible to introduce quotes via the AddTextRaw
-		// method, this function should replace " with \" when appropriate.
-		// FIXME: This doesn't really do the right thing when we have property queries.
-		[XmlIgnore]
-		public string QuotedText {
-			get { 
-				StringBuilder builder = new StringBuilder ();
-				
-				foreach (QueryPart part in parts) {
-					if (part.Text == null)
-						continue;
-
-					if (builder.Length > 0)
-						builder.Append (' ');
-					
-					if (part.IsRequired)
-						builder.Append ('+');
-					if (part.IsProhibited)
-						builder.Append ('-');
-
-					bool has_ws = BU.StringFu.ContainsWhiteSpace (part.Text);
-					if (has_ws)
-						builder.Append ('"');
-					builder.Append (part.Text);
-					if (has_ws)
-						builder.Append ('"');
-				}
-
-				return builder.ToString ();
-			}
-		}
-
-		[XmlArrayItem (ElementName="Part",
-			       Type=typeof (QueryPart))]
+		[XmlArrayItem (ElementName="Part", Type=typeof (QueryPart))]
 		[XmlArray (ElementName="Parts")]
 		public ArrayList Parts {
 			get { return parts; }
@@ -255,20 +178,57 @@ namespace Beagle {
 
 		[XmlIgnore]
 		public ICollection Text {
+			get { return exact_text; }
+		}
+
+		[XmlIgnore]
+		public string QuotedText {
 			get {
-				ArrayList text = new ArrayList ();
-				foreach (QueryPart part in parts)
-					if (part.Text != null)
-						text.Add (part.Text);
-				return text;
+				StringBuilder builder = new StringBuilder ();
+				foreach (string text in Text) {
+					string text_cooked = text;
+					if (builder.Length > 0)
+						builder.Append (' ');
+					bool contains_space = (text.IndexOf (' ') != -1);
+					if (contains_space) {
+						text_cooked = text.Replace ("\"", "\\\"");
+						builder.Append ('"');
+					}
+					builder.Append (text_cooked);
+					if (contains_space)
+						builder.Append ('"');
+				}
+				return builder.ToString ();
 			}
 		}
 
+		[XmlIgnore]
+		public ICollection StemmedText {
+			get { return stemmed_text; }
+		}
+						
 		///////////////////////////////////////////////////////////////
 
+		// This API is DEPRECATED.
+		// The mime type is now stored in the beagle:MimeType property.
+		// To restrict on mime type, just do a normal property query.
+
 		public void AddMimeType (string str)
 		{
 			mimeTypes.Add (str);
+
+			if (mime_type_part == null) {
+				mime_type_part = new QueryPart_Or ();
+				AddPart (mime_type_part);
+			}
+
+			// Create a part for this mime type.
+			QueryPart_Property part;
+			part = new QueryPart_Property ();
+			part.Type = PropertyType.Keyword;
+			part.Key = "beagle:MimeType";
+			part.Value = str;
+			mime_type_part.Add (part);
 		}
 
 		public bool AllowsMimeType (string str)
@@ -294,9 +254,26 @@ namespace Beagle {
 
 		///////////////////////////////////////////////////////////////
 
+		// This API is DEPRECATED.
+		// The mime type is now stored in the beagle:HitType property.
+		// To restrict on type, just do a normal property query.
+
 		public void AddHitType (string str)
 		{
 			hitTypes.Add (str);
+
+			if (hit_type_part == null) {
+				hit_type_part = new QueryPart_Or ();
+				AddPart (hit_type_part);
+			}
+
+			// Add a part for this hit type.
+			QueryPart_Property part;
+			part = new QueryPart_Property ();
+			part.Type = PropertyType.Keyword;
+			part.Key = "beagle:HitType";
+			part.Value = str;
+			hit_type_part.Add (part);
 		}
 
 		public bool AllowsHitType (string str)
@@ -323,9 +300,26 @@ namespace Beagle {
 
 		///////////////////////////////////////////////////////////////
 
+		// This API is DEPRECATED.
+		// The source is now stored in the beagle:Source property.
+		// To restrict on source, just do a normal property query.
+
 		public void AddSource (string str)
 		{
 			searchSources.Add (str);
+
+			if (source_part == null) {
+				source_part = new QueryPart_Or ();
+				AddPart (source_part);
+			}
+
+			// Add a part for this source type.
+			QueryPart_Property part;
+			part = new QueryPart_Property ();
+			part.Type = PropertyType.Keyword;
+			part.Key = "beagle:Source";
+			part.Value = str;
+			source_part.Add (part);
 		}
 		
 
@@ -334,7 +328,7 @@ namespace Beagle {
 			if (searchSources.Count == 0)
 				return true;
 			foreach (string ss in searchSources)
-				if (str.ToUpper () == ss.ToUpper ())
+				if (str == ss)
 					return true;
 			return false;
 		}
@@ -353,6 +347,11 @@ namespace Beagle {
 
 		///////////////////////////////////////////////////////////////
 
+		public QueryDomain QueryDomain {
+			get { return domainFlags; }
+			set { domainFlags = value; }
+		}
+
 		public void AddDomain (Beagle.QueryDomain d)
 		{
 			domainFlags |= d;
@@ -384,8 +383,5 @@ namespace Beagle {
 				      && mimeTypes.Count == 0
 				      && searchSources.Count == 0; }
 		}
-
-		///////////////////////////////////////////////////////////////
-
 	}
 }
