@@ -1,4 +1,5 @@
 using System;
+using System.Globalization;
 using Mono.Unix;
 
 namespace Search.Tiles {
@@ -55,6 +56,12 @@ namespace Search.Tiles {
 			return dt;
 		}
 
+		private static DateTimeFormatInfo DateTimeFormat = CultureInfo.CurrentCulture.DateTimeFormat;
+		private static string ShortMonthDayPattern = DateTimeFormat.MonthDayPattern.Replace ("MMMM", "MMM");
+		private static string ShortYearMonthPattern = DateTimeFormat.YearMonthPattern.Replace ("MMMM", "MMM");
+		private static string MonthDayPattern = DateTimeFormat.MonthDayPattern;
+		private static string LongDatePattern = DateTimeFormat.LongDatePattern.Replace ("dddd, ", "").Replace ("dddd ", "").Replace (" dddd", "");
+
 		public static string NiceShortDate (string timestamp)
 		{
 			DateTime dt;
@@ -78,15 +85,15 @@ namespace Search.Tiles {
 			TimeSpan span = today - dt;
 
 			if (span.TotalDays < 1)
-				return "Today";
+				return Catalog.GetString ("Today");
 			else if (span.TotalDays < 2)
-				return "Yesterday";
+				return Catalog.GetString ("Yesterday");
 			else if (span.TotalDays < 7)
 				return dt.ToString ("dddd"); // "Tuesday"
 			else if (dt.Year == today.Year || span.TotalDays < 180)
-				return dt.ToString ("MMM d"); // "Jul 4"
+				return dt.ToString (ShortMonthDayPattern); // "Jul 4"
 			else
-				return dt.ToString ("MMM yyyy"); // Jan 2001
+				return dt.ToString (ShortYearMonthPattern); // "Jan 2001"
 		}
 
 		public static string NiceLongDate (string timestamp)
@@ -112,15 +119,15 @@ namespace Search.Tiles {
 			TimeSpan span = today - dt;
 
 			if (span.TotalDays < 1)
-				return "Today";
+				return Catalog.GetString ("Today");
 			else if (span.TotalDays < 2)
-				return "Yesterday";
+				return Catalog.GetString ("Yesterday");
 			else if (span.TotalDays < 7)
 				return dt.ToString ("dddd"); // "Tuesday"
 			else if (dt.Year == today.Year || span.TotalDays < 180)
-				return dt.ToString ("MMMM d"); // "July 4"
+				return dt.ToString (MonthDayPattern); // "July 4"
 			else
-				return dt.ToString ("MMMM d, yyyy"); // January 7, 2001
+				return dt.ToString (LongDatePattern); // January 7, 2001
 		}
 
 		public static string NiceVeryLongDate (string timestamp)
@@ -146,14 +153,14 @@ namespace Search.Tiles {
 			TimeSpan span = today - dt;
 
 			if (span.TotalDays < 1)
-				return String.Format ("Today ({0:MMMM d, yyyy})", dt);
+				return Catalog.GetString ("Today");
 			else if (span.TotalDays < 2)
-				return String.Format ("Yesterday ({0:MMMM d, yyyy})", dt);
+				return Catalog.GetString ("Yesterday");
 			else if (span.TotalDays < 7)
-				return String.Format ("{0:dddd} ({0:MMMM d, yyyy})", dt);
+				return dt.ToString ("dddd"); // "Tuesday"
 			else if (span.TotalDays < 30)
 				return String.Format (Catalog.GetPluralString ("{0} week ago", "{0} weeks ago", span.Days / 7) + " ({1:MMMM d, yyyy})", span.Days / 7, dt);
-			else if (span.TotalDays < 365 + 180) // Lets say a year and a half to stop saying months
+			else if (span.TotalDays < 365 + 180) // Let's say a year and a half to stop saying months
 				return String.Format (Catalog.GetPluralString ("{0} month ago", "{0} months ago", span.Days / 30) + " ({1:MMMM d, yyyy})", span.Days / 30, dt);
 			else
 				return String.Format (Catalog.GetPluralString ("{0} year ago", "{0} years ago", span.Days / 365) + " ({1:MMMM d, yyyy})", span.Days / 365, dt);
