@@ -23,9 +23,12 @@ namespace Search {
 
 	public class UIManager : Gtk.UIManager {
 
+		private Gtk.ActionGroup actions;
+		private Gtk.RadioActionEntry[] scope_entries, sort_entries;
+
 		public UIManager (Gtk.Window mainWindow)
 		{
-			Gtk.ActionGroup actions = new ActionGroup ("Actions");
+			actions = new ActionGroup ("Actions");
 
 			Gtk.ActionEntry[] entries = new ActionEntry[] {
 				new ActionEntry ("Search", null,
@@ -63,11 +66,14 @@ namespace Search {
 						 null, null, QuickTips),
 				new ActionEntry ("FocusSearchEntry", null, "",
 						 "<control>K", null,
+						 OnFocusSearchEntry),
+				new ActionEntry ("FocusSearchEntry2", null, "",
+						 "<control>L", null,
 						 OnFocusSearchEntry)
 			};
 			actions.Add (entries);
 
-			Gtk.RadioActionEntry[] scope_entries = new RadioActionEntry[] {
+			scope_entries = new RadioActionEntry[] {
 				new RadioActionEntry ("Everywhere", null,
 						      Catalog.GetString ("_Everywhere"),
 						      "<control>E",
@@ -106,7 +112,7 @@ namespace Search {
 			};
 			actions.Add (scope_entries, (int)ScopeType.Everywhere, OnScopeChanged);
 
-			Gtk.RadioActionEntry[] sort_entries = new RadioActionEntry[] {
+			sort_entries = new RadioActionEntry[] {
 				new RadioActionEntry ("Modified", null,
 						      Catalog.GetString ("Date _Modified"), null,
 						      Catalog.GetString ("Sort the most-recently-modified matches first"),
@@ -133,6 +139,22 @@ namespace Search {
 			}
 		}
 
+		private bool sensitive = true;
+		public bool Sensitive {
+			get { return this.sensitive; }
+			set {
+				this.sensitive = value;
+
+				actions ["QuickTips"].Sensitive = value;
+
+				foreach (Gtk.RadioActionEntry rae in scope_entries)
+					actions [rae.name].Sensitive = value;
+
+				foreach (Gtk.RadioActionEntry rae in sort_entries)
+					actions [rae.name].Sensitive = value;
+			}
+		}
+
 		private const string ui_def =
 		"<ui>" +
 		"  <menubar name='MenuBar'>" +
@@ -162,6 +184,7 @@ namespace Search {
 		"    </menu>" +
 		"  </menubar>" +
 		"  <accelerator action='FocusSearchEntry'/>" +
+		"  <accelerator action='FocusSearchEntry2'/>" +
 		"</ui>";
 
 		private void Preferences (object obj, EventArgs args)
@@ -200,16 +223,22 @@ namespace Search {
 		{
 			Gdk.Pixbuf logo = Beagle.Images.GetPixbuf ("system-search.png");
 
-			string[] people = new string[] {"Dan Winship <danw@novell.com>", "Lukas Lipka <lukas@pmad.net>",
-		       					 "Fredrik Hedberg <fredrik@avafan.com>", "Joe Shaw <joeshaw@novell.com>"};
-			string[] documentors = new string[] {""};
+			string[] people = new string[] { "Anna Dirks <anna@novell.com>",
+							 "Fredrik Hedberg <fredrik@avafan.com>",
+							 "Lukas Lipka <lukas@pmad.net>",
+							 "Joe Shaw <joeshaw@novell.com>", 
+							 "Jakub Steiner <jimmac@novell.com>",
+							 "Dan Winship <danw@novell.com>" };
 			
-			Gnome.About about = new Gnome.About ("Desktop Search", "0.0",
-							     "Copyright 2005 Novell, Inc.",
-							     null, people, documentors, null,
+#pragma warning disable 612 // don't warn that Gnome.About is deprecated
+			Gnome.About about = new Gnome.About ("Beagle Search",
+							     Beagle.Util.ExternalStringsHack.Version,
+							     "Copyright 2005-2006 Novell, Inc.",
+							     null, people, null, null,
 							     logo);
 			about.Run ();
 			about.Dispose ();
+#pragma warning restore 612
 		}
 
 		private void OnFocusSearchEntry (object obj, EventArgs args)
